# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class MultiplyScalarVolumesInputSpec(CommandLineInputSpec):
    inputVolume1 = File(
        position=-3, desc="Input volume 1", exists=True, argstr="%s")
    inputVolume2 = File(
        position=-2, desc="Input volume 2", exists=True, argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Volume1 * Volume2",
        argstr="%s")
    order = traits.Enum(
        "0",
        "1",
        "2",
        "3",
        desc=
        "Interpolation order if two images are in different coordinate frames or have different sampling.",
        argstr="--order %s")


class MultiplyScalarVolumesOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Volume1 * Volume2", exists=True)


class MultiplyScalarVolumes(SEMLikeCommandLine):
    """title: Multiply Scalar Volumes

category: Filtering.Arithmetic

description: Multiplies two images. Although all image types are supported on input, only signed types are produced. The two images do not have to have the same dimensions.

version: 0.1.0.$Revision: 8595 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/Multiply

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = MultiplyScalarVolumesInputSpec
    output_spec = MultiplyScalarVolumesOutputSpec
    _cmd = "MultiplyScalarVolumes "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}


class MaskScalarVolumeInputSpec(CommandLineInputSpec):
    InputVolume = File(
        position=-3,
        desc="Input volume to be masked",
        exists=True,
        argstr="%s")
    MaskVolume = File(
        position=-2,
        desc="Label volume containing the mask",
        exists=True,
        argstr="%s")
    OutputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc=
        "Output volume: Input Volume masked by label value from Mask Volume",
        argstr="%s")
    label = traits.Int(
        desc="Label value in the Mask Volume to use as the mask",
        argstr="--label %d")
    replace = traits.Int(
        desc="Value to use for the output volume outside of the mask",
        argstr="--replace %d")


class MaskScalarVolumeOutputSpec(TraitedSpec):
    OutputVolume = File(
        position=-1,
        desc=
        "Output volume: Input Volume masked by label value from Mask Volume",
        exists=True)


class MaskScalarVolume(SEMLikeCommandLine):
    """title: Mask Scalar Volume

category: Filtering.Arithmetic

description: Masks two images. The output image is set to 0 everywhere except where the chosen label from the mask volume is present, at which point it will retain it's original values. Although all image types are supported on input, only signed types are produced. The two images do not have to have the same dimensions.

version: 0.1.0.$Revision: 8595 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/Mask

contributor: Nicole Aucoin (SPL, BWH), Ron Kikinis (SPL, BWH)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = MaskScalarVolumeInputSpec
    output_spec = MaskScalarVolumeOutputSpec
    _cmd = "MaskScalarVolume "
    _outputs_filenames = {'OutputVolume': 'OutputVolume.nii'}


class SubtractScalarVolumesInputSpec(CommandLineInputSpec):
    inputVolume1 = File(
        position=-3, desc="Input volume 1", exists=True, argstr="%s")
    inputVolume2 = File(
        position=-2, desc="Input volume 2", exists=True, argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Volume1 - Volume2",
        argstr="%s")
    order = traits.Enum(
        "0",
        "1",
        "2",
        "3",
        desc=
        "Interpolation order if two images are in different coordinate frames or have different sampling.",
        argstr="--order %s")


class SubtractScalarVolumesOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Volume1 - Volume2", exists=True)


class SubtractScalarVolumes(SEMLikeCommandLine):
    """title: Subtract Scalar Volumes

category: Filtering.Arithmetic

description: Subtracts two images. Although all image types are supported on input, only signed types are produced. The two images do not have to have the same dimensions.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/Subtract

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = SubtractScalarVolumesInputSpec
    output_spec = SubtractScalarVolumesOutputSpec
    _cmd = "SubtractScalarVolumes "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}


class AddScalarVolumesInputSpec(CommandLineInputSpec):
    inputVolume1 = File(
        position=-3, desc="Input volume 1", exists=True, argstr="%s")
    inputVolume2 = File(
        position=-2, desc="Input volume 2", exists=True, argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Volume1 + Volume2",
        argstr="%s")
    order = traits.Enum(
        "0",
        "1",
        "2",
        "3",
        desc=
        "Interpolation order if two images are in different coordinate frames or have different sampling.",
        argstr="--order %s")


class AddScalarVolumesOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Volume1 + Volume2", exists=True)


class AddScalarVolumes(SEMLikeCommandLine):
    """title: Add Scalar Volumes

category: Filtering.Arithmetic

description: Adds two images. Although all image types are supported on input, only signed types are produced. The two images do not have to have the same dimensions.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/Add

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = AddScalarVolumesInputSpec
    output_spec = AddScalarVolumesOutputSpec
    _cmd = "AddScalarVolumes "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}


class CastScalarVolumeInputSpec(CommandLineInputSpec):
    InputVolume = File(
        position=-2,
        desc="Input volume, the volume to cast.",
        exists=True,
        argstr="%s")
    OutputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output volume, cast to the new type.",
        argstr="%s")
    type = traits.Enum(
        "Char",
        "UnsignedChar",
        "Short",
        "UnsignedShort",
        "Int",
        "UnsignedInt",
        "Float",
        "Double",
        desc="Type for the new output volume.",
        argstr="--type %s")


class CastScalarVolumeOutputSpec(TraitedSpec):
    OutputVolume = File(
        position=-1, desc="Output volume, cast to the new type.", exists=True)


class CastScalarVolume(SEMLikeCommandLine):
    """title: Cast Scalar Volume

category: Filtering.Arithmetic

description: Cast a volume to a given data type.
Use at your own risk when casting an input volume into a lower precision type!
Allows casting to the same type as the input volume.

version: 0.1.0.$Revision: 2104 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/Cast

contributor: Nicole Aucoin (SPL, BWH), Ron Kikinis (SPL, BWH)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = CastScalarVolumeInputSpec
    output_spec = CastScalarVolumeOutputSpec
    _cmd = "CastScalarVolume "
    _outputs_filenames = {'OutputVolume': 'OutputVolume.nii'}



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Check the resolving/rebasing feature of ``BasePath``s."""
from __future__ import print_function, unicode_literals

from ... import base as nib
from ..traits_extension import rebase_path_traits, resolve_path_traits, Path


class _test_spec(nib.TraitedSpec):
    a = nib.File()
    b = nib.traits.Tuple(nib.File(), nib.File())
    c = nib.traits.List(nib.File())
    d = nib.traits.Either(nib.File(), nib.traits.Float())
    e = nib.OutputMultiObject(nib.File())
    ee = nib.OutputMultiObject(nib.Str)
    f = nib.traits.Dict(nib.Str, nib.File())
    g = nib.traits.Either(nib.File, nib.Str)
    h = nib.Str
    i = nib.traits.Either(nib.File, nib.traits.Tuple(nib.File, nib.traits.Int))
    j = nib.traits.Either(nib.File, nib.traits.Tuple(nib.File, nib.traits.Int),
                          nib.traits.Dict(nib.Str, nib.File()))
    k = nib.DictStrStr


def test_rebase_resolve_path_traits():
    """Check rebase_path_traits and resolve_path_traits and idempotence."""
    spec = _test_spec()

    v = '/some/path/f1.txt'
    a = rebase_path_traits(spec.trait('a'), v, '/some/path')
    assert a == Path('f1.txt')

    # Idempotence
    assert rebase_path_traits(spec.trait('a'), a, '/some/path') == a

    a = resolve_path_traits(spec.trait('a'), a, '/some/path')
    assert a == Path(v)

    # Idempotence
    assert resolve_path_traits(spec.trait('a'), a, '/some/path') == a

    a = rebase_path_traits(spec.trait('a'), v, '/some/other/path')
    assert a == Path(v)

    # Idempotence
    assert rebase_path_traits(spec.trait('a'), a, '/some/other/path') == a

    a = resolve_path_traits(spec.trait('a'), a, '/some/path')
    assert a == Path(v)

    # Idempotence
    assert resolve_path_traits(spec.trait('a'), a, '/some/path') == a

    v = ('/some/path/f1.txt', '/some/path/f2.txt')
    b = rebase_path_traits(spec.trait('b'), v, '/some/path')
    assert b == (Path('f1.txt'), Path('f2.txt'))

    # Idempotence
    assert rebase_path_traits(spec.trait('b'), b, '/some/path') == b

    b = resolve_path_traits(spec.trait('b'), b, '/some/path')
    assert b == (Path(v[0]), Path(v[1]))

    # Idempotence
    assert resolve_path_traits(spec.trait('b'), b, '/some/path') == b

    v = ['/some/path/f1.txt', '/some/path/f2.txt', '/some/path/f3.txt']
    c = rebase_path_traits(spec.trait('c'), v, '/some/path')
    assert c == [Path('f1.txt'), Path('f2.txt'), Path('f3.txt')]

    # Idempotence
    assert rebase_path_traits(spec.trait('c'), c, '/some/path') == c

    c = resolve_path_traits(spec.trait('c'), c, '/some/path')
    assert c == [Path(vp) for vp in v]

    # Idempotence
    assert resolve_path_traits(spec.trait('c'), c, '/some/path') == c

    v = 2.0
    d = rebase_path_traits(spec.trait('d'), v, '/some/path')
    assert d == v

    d = resolve_path_traits(spec.trait('d'), d, '/some/path')
    assert d == v

    v = '/some/path/either.txt'
    d = rebase_path_traits(spec.trait('d'), v, '/some/path')
    assert d == Path('either.txt')

    # Idempotence
    assert rebase_path_traits(spec.trait('d'), d, '/some/path') == d

    d = resolve_path_traits(spec.trait('d'), d, '/some/path')
    assert d == Path(v)

    # Idempotence
    assert resolve_path_traits(spec.trait('d'), d, '/some/path') == d

    v = ['/some/path/f1.txt', '/some/path/f2.txt', '/some/path/f3.txt']
    e = rebase_path_traits(spec.trait('e'), v, '/some/path')
    assert e == [Path('f1.txt'), Path('f2.txt'), Path('f3.txt')]

    # Idempotence
    assert rebase_path_traits(spec.trait('e'), e, '/some/path') == e

    e = resolve_path_traits(spec.trait('e'), e, '/some/path')
    assert e == [Path(vp) for vp in v]

    # Idempotence
    assert resolve_path_traits(spec.trait('e'), e, '/some/path') == e

    v = [['/some/path/f1.txt', '/some/path/f2.txt'], [['/some/path/f3.txt']]]
    e = rebase_path_traits(spec.trait('e'), v, '/some/path')
    assert e == [[Path('f1.txt'), Path('f2.txt')], [[Path('f3.txt')]]]

    # Idempotence
    assert rebase_path_traits(spec.trait('e'), e, '/some/path') == e

    e = resolve_path_traits(spec.trait('e'), e, '/some/path')
    assert e == [[[Path(vpp) for vpp in vp] if isinstance(vp, list) else Path(vp) for vp in inner]
                 for inner in v]

    # Idempotence
    assert resolve_path_traits(spec.trait('e'), e, '/some/path') == e

    # These are Str - no rebasing/resolving should happen
    v = [['/some/path/f1.txt', '/some/path/f2.txt'], [['/some/path/f3.txt']]]
    ee = rebase_path_traits(spec.trait('ee'), v, '/some/path')
    assert ee == v

    # Idempotence
    assert rebase_path_traits(spec.trait('ee'), ee, '/some/path') == ee

    ee = resolve_path_traits(spec.trait('ee'), [['f1.txt', 'f2.txt'], [['f3.txt']]], '/some/path')
    assert ee == [['f1.txt', 'f2.txt'], [['f3.txt']]]

    # Idempotence
    assert resolve_path_traits(spec.trait('ee'), ee, '/some/path') == ee

    v = {'1': '/some/path/f1.txt'}
    f = rebase_path_traits(spec.trait('f'), v, '/some')
    assert f == {'1': Path('path/f1.txt')}

    # Idempotence
    assert rebase_path_traits(spec.trait('f'), f, '/some') == f

    f = resolve_path_traits(spec.trait('f'), f, '/some')
    assert f == {k: Path(val) for k, val in v.items()}

    # Idempotence
    assert resolve_path_traits(spec.trait('f'), f, '/some') == f

    # Either(Str, File): passing in path-like apply manipulation
    v = '/some/path/either.txt'
    g = rebase_path_traits(spec.trait('g'), v, '/some/path')
    assert g == Path('either.txt')

    # Idempotence
    assert rebase_path_traits(spec.trait('g'), g, '/some/path') == g

    g = resolve_path_traits(spec.trait('g'), g, '/some/path')
    assert g == Path(v)

    # Idempotence
    assert resolve_path_traits(spec.trait('g'), g, '/some/path') == g

    g = rebase_path_traits(spec.trait('g'), v, '/some')
    assert g == Path('path/either.txt')

    # Idempotence
    assert rebase_path_traits(spec.trait('g'), g, '/some/path') == g

    g = resolve_path_traits(spec.trait('g'), g, '/some')
    assert g == Path(v)

    # Idempotence
    assert resolve_path_traits(spec.trait('g'), g, '/some/path') == g

    # Either(Str, File): passing str discards File
    v = 'either.txt'
    g = rebase_path_traits(spec.trait('g'), v, '/some/path')
    assert g == v

    # Idempotence
    assert rebase_path_traits(spec.trait('g'), g, '/some/path') == g

    # This is a problematic case, it is impossible to know whether this
    # was meant to be a string or a file.
    # In this implementation, strings take precedence
    g = resolve_path_traits(spec.trait('g'), g, '/some/path')
    assert g == v

    # Idempotence
    assert resolve_path_traits(spec.trait('g'), g, '/some/path') == g

    v = 'string'
    g = rebase_path_traits(spec.trait('g'), v, '/some')
    assert g == v

    # Idempotence
    assert rebase_path_traits(spec.trait('g'), g, '/some') == g

    # This is a problematic case, it is impossible to know whether this
    # was meant to be a string or a file.
    g = resolve_path_traits(spec.trait('g'), v, '/some')
    assert g == v

    # Idempotence
    assert resolve_path_traits(spec.trait('g'), g, '/some') == g

    g = rebase_path_traits(spec.trait('g'), v, '/some/path')
    assert g == v  # You dont want this one to be a Path

    # Idempotence
    assert rebase_path_traits(spec.trait('g'), g, '/some/path') == g

    # This is a problematic case, it is impossible to know whether this
    # was meant to be a string or a file.
    g = resolve_path_traits(spec.trait('g'), g, '/some/path')
    assert g == v  # You dont want this one to be a Path

    # Idempotence
    assert resolve_path_traits(spec.trait('g'), g, '/some/path') == g

    h = rebase_path_traits(spec.trait('h'), v, '/some/path')
    assert h == v

    # Idempotence
    assert rebase_path_traits(spec.trait('h'), h, '/some/path') == h

    h = resolve_path_traits(spec.trait('h'), h, '/some/path')
    assert h == v

    # Idempotence
    assert resolve_path_traits(spec.trait('h'), h, '/some/path') == h

    v = '/some/path/either/file.txt'
    i = rebase_path_traits(spec.trait('i'), v, '/some/path')
    assert i == Path('either/file.txt')

    # Idempotence
    assert rebase_path_traits(spec.trait('i'), i, '/some/path') == i

    i = resolve_path_traits(spec.trait('i'), i, '/some/path')
    assert i == Path(v)

    # Idempotence
    assert resolve_path_traits(spec.trait('i'), i, '/some/path') == i

    v = ('/some/path/either/tuple/file.txt', 2)
    i = rebase_path_traits(spec.trait('i'), v, '/some/path')
    assert i == (Path('either/tuple/file.txt'), 2)

    # Idempotence
    assert rebase_path_traits(spec.trait('i'), i, '/some/path') == i

    i = resolve_path_traits(spec.trait('i'), i, '/some/path')
    assert i == (Path(v[0]), v[1])

    # Idempotence
    assert resolve_path_traits(spec.trait('i'), i, '/some/path') == i

    v = '/some/path/either/file.txt'
    j = rebase_path_traits(spec.trait('j'), v, '/some/path')
    assert j == Path('either/file.txt')

    # Idempotence
    assert rebase_path_traits(spec.trait('j'), j, '/some/path') == j

    j = resolve_path_traits(spec.trait('j'), j, '/some/path')
    assert j == Path(v)

    # Idempotence
    assert resolve_path_traits(spec.trait('j'), j, '/some/path') == j

    v = ('/some/path/either/tuple/file.txt', 2)
    j = rebase_path_traits(spec.trait('j'), ('/some/path/either/tuple/file.txt', 2), '/some/path')
    assert j == (Path('either/tuple/file.txt'), 2)

    # Idempotence
    assert rebase_path_traits(spec.trait('j'), j, '/some/path') == j

    j = resolve_path_traits(spec.trait('j'), j, '/some/path')
    assert j == (Path(v[0]), v[1])

    # Idempotence
    assert resolve_path_traits(spec.trait('j'), j, '/some/path') == j

    v = {'a': '/some/path/either/dict/file.txt'}
    j = rebase_path_traits(spec.trait('j'), v, '/some/path')
    assert j == {'a': Path('either/dict/file.txt')}

    # Idempotence
    assert rebase_path_traits(spec.trait('j'), j, '/some/path') == j

    j = resolve_path_traits(spec.trait('j'), j, '/some/path')
    assert j == {k: Path(val) for k, val in v.items()}

    # Idempotence
    assert resolve_path_traits(spec.trait('j'), j, '/some/path') == j

    v = {'path': '/some/path/f1.txt'}
    k = rebase_path_traits(spec.trait('k'), v, '/some/path')
    assert k == v

    # Idempotence
    assert rebase_path_traits(spec.trait('k'), k, '/some/path') == k

    k = resolve_path_traits(spec.trait('k'), k, '/some/path')
    assert k == v



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..calib import SFPICOCalibData


def test_SFPICOCalibData_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        info_file=dict(
            argstr='-infooutputfile %s',
            extensions=None,
            genfile=True,
            hash_files=False,
            mandatory=True,
        ),
        onedtfarange=dict(
            argstr='-onedtfarange %s',
            units='NA',
        ),
        onedtfastep=dict(
            argstr='-onedtfastep %f',
            units='NA',
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        scheme_file=dict(
            argstr='-schemefile %s',
            extensions=None,
            mandatory=True,
        ),
        seed=dict(
            argstr='-seed %f',
            units='NA',
        ),
        snr=dict(
            argstr='-snr %f',
            units='NA',
        ),
        trace=dict(
            argstr='-trace %f',
            units='NA',
        ),
        twodtanglerange=dict(
            argstr='-twodtanglerange %s',
            units='NA',
        ),
        twodtanglestep=dict(
            argstr='-twodtanglestep %f',
            units='NA',
        ),
        twodtfarange=dict(
            argstr='-twodtfarange %s',
            units='NA',
        ),
        twodtfastep=dict(
            argstr='-twodtfastep %f',
            units='NA',
        ),
        twodtmixmax=dict(
            argstr='-twodtmixmax %f',
            units='NA',
        ),
        twodtmixstep=dict(
            argstr='-twodtmixstep %f',
            units='NA',
        ),
    )
    inputs = SFPICOCalibData.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SFPICOCalibData_outputs():
    output_map = dict(
        PICOCalib=dict(extensions=None, ),
        calib_info=dict(extensions=None, ),
    )
    outputs = SFPICOCalibData.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..c3 import C3dAffineTool


def test_C3dAffineTool_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fsl2ras=dict(
            argstr='-fsl2ras',
            position=4,
        ),
        itk_transform=dict(
            argstr='-oitk %s',
            hash_files=False,
            position=5,
        ),
        reference_file=dict(
            argstr='-ref %s',
            extensions=None,
            position=1,
        ),
        source_file=dict(
            argstr='-src %s',
            extensions=None,
            position=2,
        ),
        transform_file=dict(
            argstr='%s',
            extensions=None,
            position=3,
        ),
    )
    inputs = C3dAffineTool.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_C3dAffineTool_outputs():
    output_map = dict(itk_transform=dict(extensions=None, ), )
    outputs = C3dAffineTool.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Normalize


def test_Normalize_inputs():
    input_map = dict(
        DCT_period_cutoff=dict(field='eoptions.cutoff', ),
        affine_regularization_type=dict(field='eoptions.regtype', ),
        apply_to_files=dict(
            copyfile=True,
            field='subj.resample',
        ),
        jobtype=dict(usedefault=True, ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        nonlinear_iterations=dict(field='eoptions.nits', ),
        nonlinear_regularization=dict(field='eoptions.reg', ),
        out_prefix=dict(
            field='roptions.prefix',
            usedefault=True,
        ),
        parameter_file=dict(
            copyfile=False,
            extensions=None,
            field='subj.matname',
            mandatory=True,
            xor=['source', 'template'],
        ),
        paths=dict(),
        source=dict(
            copyfile=True,
            field='subj.source',
            mandatory=True,
            xor=['parameter_file'],
        ),
        source_image_smoothing=dict(field='eoptions.smosrc', ),
        source_weight=dict(
            copyfile=False,
            extensions=None,
            field='subj.wtsrc',
        ),
        template=dict(
            copyfile=False,
            extensions=None,
            field='eoptions.template',
            mandatory=True,
            xor=['parameter_file'],
        ),
        template_image_smoothing=dict(field='eoptions.smoref', ),
        template_weight=dict(
            copyfile=False,
            extensions=None,
            field='eoptions.weight',
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        write_bounding_box=dict(field='roptions.bb', ),
        write_interp=dict(field='roptions.interp', ),
        write_preserve=dict(field='roptions.preserve', ),
        write_voxel_sizes=dict(field='roptions.vox', ),
        write_wrap=dict(field='roptions.wrap', ),
    )
    inputs = Normalize.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Normalize_outputs():
    output_map = dict(
        normalization_parameters=dict(),
        normalized_files=dict(),
        normalized_source=dict(),
    )
    outputs = Normalize.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ApplyInverseDeformation


def test_ApplyInverseDeformation_inputs():
    input_map = dict(
        bounding_box=dict(field='comp{1}.inv.comp{1}.sn2def.bb', ),
        deformation=dict(
            extensions=None,
            field='comp{1}.inv.comp{1}.sn2def.matname',
            xor=['deformation_field'],
        ),
        deformation_field=dict(
            extensions=None,
            field='comp{1}.inv.comp{1}.def',
            xor=['deformation'],
        ),
        in_files=dict(
            field='fnames',
            mandatory=True,
        ),
        interpolation=dict(field='interp', ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        paths=dict(),
        target=dict(
            extensions=None,
            field='comp{1}.inv.space',
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        voxel_sizes=dict(field='comp{1}.inv.comp{1}.sn2def.vox', ),
    )
    inputs = ApplyInverseDeformation.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ApplyInverseDeformation_outputs():
    output_map = dict(out_files=dict(), )
    outputs = ApplyInverseDeformation.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import TrackBedpostxProba


def test_TrackBedpostxProba_inputs():
    input_map = dict(
        anisfile=dict(
            argstr='-anisfile %s',
            extensions=None,
        ),
        anisthresh=dict(argstr='-anisthresh %f', ),
        args=dict(argstr='%s', ),
        bedpostxdir=dict(
            argstr='-bedpostxdir %s',
            mandatory=True,
        ),
        curveinterval=dict(
            argstr='-curveinterval %f',
            requires=['curvethresh'],
        ),
        curvethresh=dict(argstr='-curvethresh %f', ),
        data_dims=dict(
            argstr='-datadims %s',
            units='voxels',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        gzip=dict(argstr='-gzip', ),
        in_file=dict(
            argstr='-inputfile %s',
            extensions=None,
            position=1,
        ),
        inputdatatype=dict(argstr='-inputdatatype %s', ),
        inputmodel=dict(
            argstr='-inputmodel %s',
            usedefault=True,
        ),
        interpolator=dict(argstr='-interpolator %s', ),
        ipthresh=dict(argstr='-ipthresh %f', ),
        iterations=dict(
            argstr='-iterations %d',
            units='NA',
        ),
        maxcomponents=dict(
            argstr='-maxcomponents %d',
            units='NA',
        ),
        min_vol_frac=dict(
            argstr='-bedpostxminf %d',
            units='NA',
        ),
        numpds=dict(
            argstr='-numpds %d',
            units='NA',
        ),
        out_file=dict(
            argstr='-outputfile %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        output_root=dict(
            argstr='-outputroot %s',
            extensions=None,
            position=-1,
        ),
        outputtracts=dict(argstr='-outputtracts %s', ),
        seed_file=dict(
            argstr='-seedfile %s',
            extensions=None,
            position=2,
        ),
        stepsize=dict(
            argstr='-stepsize %f',
            requires=['tracker'],
        ),
        tracker=dict(
            argstr='-tracker %s',
            usedefault=True,
        ),
        voxel_dims=dict(
            argstr='-voxeldims %s',
            units='mm',
        ),
    )
    inputs = TrackBedpostxProba.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_TrackBedpostxProba_outputs():
    output_map = dict(tracked=dict(extensions=None, ), )
    outputs = TrackBedpostxProba.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import XfmAvg


def test_XfmAvg_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        avg_linear=dict(argstr='-avg_linear', ),
        avg_nonlinear=dict(argstr='-avg_nonlinear', ),
        clobber=dict(
            argstr='-clobber',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        ignore_linear=dict(argstr='-ignore_linear', ),
        ignore_nonlinear=dict(argstr='-ignore_nonline', ),
        input_files=dict(
            argstr='%s',
            mandatory=True,
            position=-2,
            sep=' ',
        ),
        input_grid_files=dict(),
        output_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        verbose=dict(argstr='-verbose', ),
    )
    inputs = XfmAvg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_XfmAvg_outputs():
    output_map = dict(
        output_file=dict(extensions=None, ),
        output_grid=dict(extensions=None, ),
    )
    outputs = XfmAvg.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import ModelFit


def test_ModelFit_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bgmask=dict(
            argstr='-bgmask %s',
            extensions=None,
        ),
        bgthresh=dict(argstr='-bgthresh %G', ),
        cfthresh=dict(argstr='-csfthresh %G', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixedbvalue=dict(argstr='-fixedbvalue %s', ),
        fixedmodq=dict(argstr='-fixedmod %s', ),
        in_file=dict(
            argstr='-inputfile %s',
            extensions=None,
            mandatory=True,
        ),
        inputdatatype=dict(argstr='-inputdatatype %s', ),
        model=dict(
            argstr='-model %s',
            mandatory=True,
        ),
        noisemap=dict(
            argstr='-noisemap %s',
            extensions=None,
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        outlier=dict(
            argstr='-outliermap %s',
            extensions=None,
        ),
        outputfile=dict(
            argstr='-outputfile %s',
            extensions=None,
        ),
        residualmap=dict(
            argstr='-residualmap %s',
            extensions=None,
        ),
        scheme_file=dict(
            argstr='-schemefile %s',
            extensions=None,
            mandatory=True,
        ),
        sigma=dict(argstr='-sigma %G', ),
        tau=dict(argstr='-tau %G', ),
    )
    inputs = ModelFit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ModelFit_outputs():
    output_map = dict(fitted_data=dict(extensions=None, ), )
    outputs = ModelFit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import MedicAlgorithmSPECTRE2010


def test_MedicAlgorithmSPECTRE2010_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inApply=dict(argstr='--inApply %s', ),
        inAtlas=dict(
            argstr='--inAtlas %s',
            extensions=None,
        ),
        inBackground=dict(argstr='--inBackground %f', ),
        inCoarse=dict(argstr='--inCoarse %f', ),
        inCost=dict(argstr='--inCost %s', ),
        inDegrees=dict(argstr='--inDegrees %s', ),
        inFind=dict(argstr='--inFind %s', ),
        inFine=dict(argstr='--inFine %f', ),
        inImage=dict(argstr='--inImage %s', ),
        inInhomogeneity=dict(argstr='--inInhomogeneity %s', ),
        inInitial=dict(argstr='--inInitial %d', ),
        inInitial2=dict(argstr='--inInitial2 %f', ),
        inInput=dict(
            argstr='--inInput %s',
            extensions=None,
        ),
        inMMC=dict(argstr='--inMMC %d', ),
        inMMC2=dict(argstr='--inMMC2 %d', ),
        inMaximum=dict(argstr='--inMaximum %f', ),
        inMinimum=dict(argstr='--inMinimum %f', ),
        inMinimum2=dict(argstr='--inMinimum2 %f', ),
        inMultiple=dict(argstr='--inMultiple %d', ),
        inMultithreading=dict(argstr='--inMultithreading %s', ),
        inNumber=dict(argstr='--inNumber %d', ),
        inNumber2=dict(argstr='--inNumber2 %d', ),
        inOutput=dict(argstr='--inOutput %s', ),
        inOutput2=dict(argstr='--inOutput2 %s', ),
        inOutput3=dict(argstr='--inOutput3 %s', ),
        inOutput4=dict(argstr='--inOutput4 %s', ),
        inOutput5=dict(argstr='--inOutput5 %s', ),
        inRegistration=dict(argstr='--inRegistration %s', ),
        inResample=dict(argstr='--inResample %s', ),
        inRun=dict(argstr='--inRun %s', ),
        inSkip=dict(argstr='--inSkip %s', ),
        inSmoothing=dict(argstr='--inSmoothing %f', ),
        inSubsample=dict(argstr='--inSubsample %s', ),
        inUse=dict(argstr='--inUse %s', ),
        null=dict(argstr='--null %s', ),
        outFANTASM=dict(
            argstr='--outFANTASM %s',
            hash_files=False,
        ),
        outMask=dict(
            argstr='--outMask %s',
            hash_files=False,
        ),
        outMidsagittal=dict(
            argstr='--outMidsagittal %s',
            hash_files=False,
        ),
        outOriginal=dict(
            argstr='--outOriginal %s',
            hash_files=False,
        ),
        outPrior=dict(
            argstr='--outPrior %s',
            hash_files=False,
        ),
        outSegmentation=dict(
            argstr='--outSegmentation %s',
            hash_files=False,
        ),
        outSplitHalves=dict(
            argstr='--outSplitHalves %s',
            hash_files=False,
        ),
        outStripped=dict(
            argstr='--outStripped %s',
            hash_files=False,
        ),
        outd0=dict(
            argstr='--outd0 %s',
            hash_files=False,
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = MedicAlgorithmSPECTRE2010.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MedicAlgorithmSPECTRE2010_outputs():
    output_map = dict(
        outFANTASM=dict(extensions=None, ),
        outMask=dict(extensions=None, ),
        outMidsagittal=dict(extensions=None, ),
        outOriginal=dict(extensions=None, ),
        outPrior=dict(extensions=None, ),
        outSegmentation=dict(extensions=None, ),
        outSplitHalves=dict(extensions=None, ),
        outStripped=dict(extensions=None, ),
        outd0=dict(extensions=None, ),
    )
    outputs = MedicAlgorithmSPECTRE2010.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brains import JointHistogram


def test_JointHistogram_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputMaskVolumeInXAxis=dict(
            argstr='--inputMaskVolumeInXAxis %s',
            extensions=None,
        ),
        inputMaskVolumeInYAxis=dict(
            argstr='--inputMaskVolumeInYAxis %s',
            extensions=None,
        ),
        inputVolumeInXAxis=dict(
            argstr='--inputVolumeInXAxis %s',
            extensions=None,
        ),
        inputVolumeInYAxis=dict(
            argstr='--inputVolumeInYAxis %s',
            extensions=None,
        ),
        outputJointHistogramImage=dict(
            argstr='--outputJointHistogramImage %s', ),
        verbose=dict(argstr='--verbose ', ),
    )
    inputs = JointHistogram.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_JointHistogram_outputs():
    output_map = dict()
    outputs = JointHistogram.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Nipype interface for seg_EM.

The em module provides higher-level interfaces to some of the operations
that can be performed with the seg_em command-line program.

Examples
--------
See the docstrings of the individual classes for examples.
"""

from ..base import (TraitedSpec, File, traits, CommandLineInputSpec,
                    InputMultiPath)
from .base import NiftySegCommand
from ..niftyreg.base import get_custom_path


class EMInputSpec(CommandLineInputSpec):
    """Input Spec for EM."""
    in_file = File(
        argstr='-in %s',
        exists=True,
        mandatory=True,
        desc='Input image to segment',
        position=4)

    mask_file = File(
        argstr='-mask %s',
        exists=True,
        desc='Filename of the ROI for label fusion')

    # Priors
    no_prior = traits.Int(
        argstr='-nopriors %s',
        mandatory=True,
        desc='Number of classes to use without prior',
        xor=['prior_4D', 'priors'])

    prior_4D = File(
        argstr='-prior4D %s',
        exists=True,
        mandatory=True,
        desc='4D file containing the priors',
        xor=['no_prior', 'priors'])

    priors = InputMultiPath(
        argstr='%s',
        mandatory=True,
        desc='List of priors filepaths.',
        xor=['no_prior', 'prior_4D'])

    # iterations
    max_iter = traits.Int(
        argstr='-max_iter %s',
        default_value=100,
        usedefault=True,
        desc='Maximum number of iterations')

    min_iter = traits.Int(
        argstr='-min_iter %s',
        default_value=0,
        usedefault=True,
        desc='Minimum number of iterations')

    # other options
    bc_order_val = traits.Int(
        argstr='-bc_order %s',
        default_value=3,
        usedefault=True,
        desc='Polynomial order for the bias field')

    mrf_beta_val = traits.Float(
        argstr='-mrf_beta %s', desc='Weight of the Markov Random Field')

    desc = 'Bias field correction will run only if the ratio of improvement \
is below bc_thresh. (default=0 [OFF])'

    bc_thresh_val = traits.Float(
        argstr='-bc_thresh %s',
        default_value=0,
        usedefault=True,
        desc=desc)

    desc = 'Amount of regularization over the diagonal of the covariance \
matrix [above 1]'

    reg_val = traits.Float(argstr='-reg %s', desc=desc)

    desc = 'Outlier detection as in (Van Leemput TMI 2003). <fl1> is the \
Mahalanobis threshold [recommended between 3 and 7] <fl2> is a convergence \
ratio below which the outlier detection is going to be done [recommended 0.01]'

    outlier_val = traits.Tuple(
        traits.Float(), traits.Float(), argstr='-outlier %s %s', desc=desc)

    desc = 'Relax Priors [relaxation factor: 0<rf<1 (recommended=0.5), \
gaussian regularization: gstd>0 (recommended=2.0)] /only 3D/'

    relax_priors = traits.Tuple(
        traits.Float(), traits.Float(), argstr='-rf %s %s', desc=desc)

    # outputs
    out_file = File(
        name_source=['in_file'],
        name_template='%s_em.nii.gz',
        argstr='-out %s',
        desc='Output segmentation')
    out_bc_file = File(
        name_source=['in_file'],
        name_template='%s_bc_em.nii.gz',
        argstr='-bc_out %s',
        desc='Output bias corrected image')
    out_outlier_file = File(
        name_source=['in_file'],
        name_template='%s_outlier_em.nii.gz',
        argstr='-out_outlier %s',
        desc='Output outlierness image')


class EMOutputSpec(TraitedSpec):
    """Output Spec for EM."""
    out_file = File(desc="Output segmentation")
    out_bc_file = File(desc="Output bias corrected image")
    out_outlier_file = File(desc='Output outlierness image')


class EM(NiftySegCommand):
    """Interface for executable seg_EM from NiftySeg platform.

    seg_EM is a general purpose intensity based image segmentation tool. In
    it's simplest form, it takes in one 2D or 3D image and segments it in n
    classes.

    `Source code <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg>`_ |
    `Documentation <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg_documentation>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyseg
    >>> node = niftyseg.EM()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.no_prior = 4
    >>> node.cmdline
    'seg_EM -in im1.nii -bc_order 3 -bc_thresh 0 -max_iter 100 -min_iter 0 -nopriors 4 \
-bc_out im1_bc_em.nii.gz -out im1_em.nii.gz -out_outlier im1_outlier_em.nii.gz'

    """
    _cmd = get_custom_path('seg_EM', env_dir='NIFTYSEGDIR')
    _suffix = '_em'
    input_spec = EMInputSpec
    output_spec = EMOutputSpec

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for seg_EM."""
        if opt == 'priors':
            _nb_priors = len(self.inputs.priors)
            return '-priors %d %s' % (_nb_priors, ' '.join(self.inputs.priors))
        else:
            return super(EM, self)._format_arg(opt, spec, val)



#!python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from io import open

import click

from .instance import list_interfaces
from .utils import (
    CONTEXT_SETTINGS,
    UNKNOWN_OPTIONS,
    ExistingDirPath,
    ExistingFilePath,
    UnexistingFilePath,
    RegularExpression,
    PythonModule,
    check_not_none,
)

from .. import __version__


# declare the CLI group
@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    pass


@cli.command(context_settings=CONTEXT_SETTINGS)
@click.argument('logdir', type=ExistingDirPath, callback=check_not_none)
@click.option(
    '-r',
    '--regex',
    type=RegularExpression(),
    callback=check_not_none,
    help='Regular expression to be searched in each traceback.')
def search(logdir, regex):
    """Search for tracebacks content.

    Search for traceback inside a folder of nipype crash log files that match
    a given regular expression.

    Examples:\n
    nipypecli search nipype/wd/log -r '.*subject123.*'
    """
    from .crash_files import iter_tracebacks

    for file, trace in iter_tracebacks(logdir):
        if regex.search(trace):
            click.echo("-" * len(file))
            click.echo(file)
            click.echo("-" * len(file))
            click.echo(trace)


@cli.command(context_settings=CONTEXT_SETTINGS)
@click.argument('crashfile', type=ExistingFilePath, callback=check_not_none)
@click.option(
    '-r', '--rerun', is_flag=True, flag_value=True, help='Rerun crashed node.')
@click.option(
    '-d',
    '--debug',
    is_flag=True,
    flag_value=True,
    help='Enable Python debugger when re-executing.')
@click.option(
    '-i',
    '--ipydebug',
    is_flag=True,
    flag_value=True,
    help='Enable IPython debugger when re-executing.')
@click.option(
    '-w',
    '--dir',
    type=ExistingDirPath,
    help='Directory where to run the node in.')
def crash(crashfile, rerun, debug, ipydebug, dir):
    """Display Nipype crash files.

    For certain crash files, one can rerun a failed node in a temp directory.

    Examples:\n
    nipypecli crash crashfile.pklz\n
    nipypecli crash crashfile.pklz -r -i\n
    """
    from .crash_files import display_crash_file

    debug = 'ipython' if ipydebug else debug
    if debug == 'ipython':
        import sys
        from IPython.core import ultratb
        sys.excepthook = ultratb.FormattedTB(
            mode='Verbose', color_scheme='Linux', call_pdb=1)
    display_crash_file(crashfile, rerun, debug, dir)


@cli.command(context_settings=CONTEXT_SETTINGS)
@click.argument('pklz_file', type=ExistingFilePath, callback=check_not_none)
def show(pklz_file):
    """Print the content of Nipype node .pklz file.

    Examples:\n
    nipypecli show node.pklz
    """
    from pprint import pprint
    from ..utils.filemanip import loadpkl

    pkl_data = loadpkl(pklz_file)
    pprint(pkl_data)


@cli.command(context_settings=UNKNOWN_OPTIONS)
@click.argument(
    'module', type=PythonModule(), required=False, callback=check_not_none)
@click.argument('interface', type=str, required=False)
@click.option(
    '--list',
    is_flag=True,
    flag_value=True,
    help='List the available Interfaces inside the given module.')
@click.option(
    '-h',
    '--help',
    is_flag=True,
    flag_value=True,
    help='Show help message and exit.')
@click.pass_context
def run(ctx, module, interface, list, help):
    """Run a Nipype Interface.

    Examples:\n
    nipypecli run nipype.interfaces.nipy --list\n
    nipypecli run nipype.interfaces.nipy ComputeMask --help
    """
    import argparse
    from .utils import add_args_options
    from ..utils.nipype_cmd import run_instance

    # print run command help if no arguments are given
    module_given = bool(module)
    if not module_given:
        click.echo(ctx.command.get_help(ctx))

    # print the list of available interfaces for the given module
    elif (module_given and list) or (module_given and not interface):
        iface_names = list_interfaces(module)
        click.echo('Available Interfaces:')
        for if_name in iface_names:
            click.echo('    {}'.format(if_name))

    # check the interface
    elif (module_given and interface):
        # create the argument parser
        description = "Run {}".format(interface)
        prog = " ".join(
            [ctx.command_path, module.__name__, interface] + ctx.args)
        iface_parser = argparse.ArgumentParser(
            description=description, prog=prog)

        # instantiate the interface
        node = getattr(module, interface)()
        iface_parser = add_args_options(iface_parser, node)

        if not ctx.args:
            # print the interface help
            try:
                iface_parser.print_help()
            except:
                print('An error ocurred when trying to print the full'
                      'command help, printing usage.')
            finally:
                iface_parser.print_usage()
        else:
            # run the interface
            args = iface_parser.parse_args(args=ctx.args)
            run_instance(node, args)


@cli.command(context_settings=CONTEXT_SETTINGS)
def version():
    """Print current version of Nipype."""
    click.echo(__version__)


@cli.group()
def convert():
    """Export nipype interfaces to other formats."""
    pass


@convert.command(context_settings=CONTEXT_SETTINGS)
@click.option(
    "-i",
    "--interface",
    type=str,
    required=True,
    help="Name of the Nipype interface to export.")
@click.option(
    "-m",
    "--module",
    type=PythonModule(),
    required=True,
    callback=check_not_none,
    help="Module where the interface is defined.")
@click.option(
    "-o",
    "--output",
    type=UnexistingFilePath,
    required=True,
    callback=check_not_none,
    help="JSON file name where the Boutiques descriptor will be "
    "written.")
@click.option(
    "-c",
    "--container-image",
    required=True,
    type=str,
    help="Name of the container image where the tool is installed.")
@click.option(
    "-p",
    "--container-type",
    required=True,
    type=str,
    help="Type of container image (Docker or Singularity).")
@click.option(
    "-x",
    "--container-index",
    type=str,
    help="Optional index where the image is available (e.g. "
    "http://index.docker.io).")
@click.option(
    "-g",
    "--ignore-inputs",
    type=str,
    multiple=True,
    help="List of interface inputs to not include in the descriptor.")
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    flag_value=True,
    help="Print information messages.")
@click.option(
    "-a",
    "--author",
    type=str,
    help="Author of the tool (required for publishing).")
@click.option(
    "-t",
    "--tags",
    type=str,
    help="JSON string containing tags to include in the descriptor,"
         "e.g. \"{\"key1\": \"value1\"}\"")
def boutiques(module, interface, container_image, container_type, output,
              container_index, verbose, author, ignore_inputs, tags):
    """Nipype to Boutiques exporter.

    See Boutiques specification at https://github.com/boutiques/schema.
    """
    from nipype.utils.nipype2boutiques import generate_boutiques_descriptor

    # Generates JSON string and saves it to file
    generate_boutiques_descriptor(
        module, interface, container_image, container_type, container_index,
        verbose, True, output, author, ignore_inputs, tags)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import PlotTimeSeries


def test_PlotTimeSeries_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            mandatory=True,
            position=1,
        ),
        labels=dict(argstr='%s', ),
        legend_file=dict(
            argstr='--legend=%s',
            extensions=None,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_type=dict(),
        plot_finish=dict(
            argstr='--finish=%d',
            xor=('plot_range', ),
        ),
        plot_range=dict(
            argstr='%s',
            xor=('plot_start', 'plot_finish'),
        ),
        plot_size=dict(argstr='%s', ),
        plot_start=dict(
            argstr='--start=%d',
            xor=('plot_range', ),
        ),
        sci_notation=dict(argstr='--sci', ),
        title=dict(argstr='%s', ),
        x_precision=dict(argstr='--precision=%d', ),
        x_units=dict(
            argstr='-u %d',
            usedefault=True,
        ),
        y_max=dict(
            argstr='--ymax=%.2f',
            xor=('y_range', ),
        ),
        y_min=dict(
            argstr='--ymin=%.2f',
            xor=('y_range', ),
        ),
        y_range=dict(
            argstr='%s',
            xor=('y_min', 'y_max'),
        ),
    )
    inputs = PlotTimeSeries.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_PlotTimeSeries_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = PlotTimeSeries.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..diffusion import TractographyLabelMapSeeding


def test_TractographyLabelMapSeeding_inputs():
    input_map = dict(
        InputVolume=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        OutputFibers=dict(
            argstr='%s',
            hash_files=False,
            position=-1,
        ),
        args=dict(argstr='%s', ),
        clthreshold=dict(argstr='--clthreshold %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputroi=dict(
            argstr='--inputroi %s',
            extensions=None,
        ),
        integrationsteplength=dict(argstr='--integrationsteplength %f', ),
        label=dict(argstr='--label %d', ),
        maximumlength=dict(argstr='--maximumlength %f', ),
        minimumlength=dict(argstr='--minimumlength %f', ),
        name=dict(argstr='--name %s', ),
        outputdirectory=dict(
            argstr='--outputdirectory %s',
            hash_files=False,
        ),
        randomgrid=dict(argstr='--randomgrid ', ),
        seedspacing=dict(argstr='--seedspacing %f', ),
        stoppingcurvature=dict(argstr='--stoppingcurvature %f', ),
        stoppingmode=dict(argstr='--stoppingmode %s', ),
        stoppingvalue=dict(argstr='--stoppingvalue %f', ),
        useindexspace=dict(argstr='--useindexspace ', ),
        writetofile=dict(argstr='--writetofile ', ),
    )
    inputs = TractographyLabelMapSeeding.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_TractographyLabelMapSeeding_outputs():
    output_map = dict(
        OutputFibers=dict(
            extensions=None,
            position=-1,
        ),
        outputdirectory=dict(),
    )
    outputs = TractographyLabelMapSeeding.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os.path as op
import numpy as np

from ... import logging
from ...utils.filemanip import split_filename
from ..base import (CommandLineInputSpec, CommandLine, BaseInterface, traits,
                    File, TraitedSpec, isdefined)
iflogger = logging.getLogger('nipype.interface')


class DWI2SphericalHarmonicsImageInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Diffusion-weighted images')
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output filename')
    encoding_file = File(
        exists=True,
        argstr='-grad %s',
        mandatory=True,
        position=1,
        desc=
        'Gradient encoding, supplied as a 4xN text file with each line is in the format [ X Y Z b ], where [ X Y Z ] describe the direction of the applied gradient, and b gives the b-value in units (1000 s/mm^2). See FSL2MRTrix'
    )
    maximum_harmonic_order = traits.Float(
        argstr='-lmax %s',
        desc=
        'set the maximum harmonic order for the output series. By default, the program will use the highest possible lmax given the number of diffusion-weighted images.'
    )
    normalise = traits.Bool(
        argstr='-normalise',
        position=3,
        desc="normalise the DW signal to the b=0 image")


class DWI2SphericalHarmonicsImageOutputSpec(TraitedSpec):
    spherical_harmonics_image = File(
        exists=True, desc='Spherical harmonics image')


class DWI2SphericalHarmonicsImage(CommandLine):
    """
    Convert base diffusion-weighted images to their spherical harmonic representation.

    This program outputs the spherical harmonic decomposition for the set measured signal attenuations.
    The signal attenuations are calculated by identifying the b-zero images from the diffusion encoding supplied
    (i.e. those with zero as the b-value), and dividing the remaining signals by the mean b-zero signal intensity.
    The spherical harmonic decomposition is then calculated by least-squares linear fitting.
    Note that this program makes use of implied symmetries in the diffusion profile.

    First, the fact the signal attenuation profile is real implies that it has conjugate symmetry,
    i.e. Y(l,-m) = Y(l,m)* (where * denotes the complex conjugate). Second, the diffusion profile should be
    antipodally symmetric (i.e. S(x) = S(-x)), implying that all odd l components should be zero. Therefore,
    this program only computes the even elements.

    Note that the spherical harmonics equations used here differ slightly from those conventionally used,
    in that the (-1)^m factor has been omitted. This should be taken into account in all subsequent calculations.

    Each volume in the output image corresponds to a different spherical harmonic component, according to the following convention:

    * [0] Y(0,0)
    * [1] Im {Y(2,2)}
    * [2] Im {Y(2,1)}
    * [3] Y(2,0)
    * [4] Re {Y(2,1)}
    * [5] Re {Y(2,2)}
    * [6] Im {Y(4,4)}
    * [7] Im {Y(4,3)}

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> dwi2SH = mrt.DWI2SphericalHarmonicsImage()
    >>> dwi2SH.inputs.in_file = 'diffusion.nii'
    >>> dwi2SH.inputs.encoding_file = 'encoding.txt'
    >>> dwi2SH.run()                                    # doctest: +SKIP
    """
    _cmd = 'dwi2SH'
    input_spec = DWI2SphericalHarmonicsImageInputSpec
    output_spec = DWI2SphericalHarmonicsImageOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['spherical_harmonics_image'] = self.inputs.out_filename
        if not isdefined(outputs['spherical_harmonics_image']):
            outputs['spherical_harmonics_image'] = op.abspath(
                self._gen_outfilename())
        else:
            outputs['spherical_harmonics_image'] = op.abspath(
                outputs['spherical_harmonics_image'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_SH.mif'


class ConstrainedSphericalDeconvolutionInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-3,
        desc='diffusion-weighted image')
    response_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc=
        'the diffusion-weighted signal response function for a single fibre population (see EstimateResponse)'
    )
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output filename')
    mask_image = File(
        exists=True,
        argstr='-mask %s',
        position=2,
        desc=
        'only perform computation within the specified binary brain mask image'
    )
    encoding_file = File(
        exists=True,
        argstr='-grad %s',
        position=1,
        desc=
        'Gradient encoding, supplied as a 4xN text file with each line is in the format [ X Y Z b ], where [ X Y Z ] describe the direction of the applied gradient, and b gives the b-value in units (1000 s/mm^2). See FSL2MRTrix'
    )
    filter_file = File(
        exists=True,
        argstr='-filter %s',
        position=-2,
        desc=
        'a text file containing the filtering coefficients for each even harmonic order.'
        'the linear frequency filtering parameters used for the initial linear spherical deconvolution step (default = [ 1 1 1 0 0 ]).'
    )

    lambda_value = traits.Float(
        argstr='-lambda %s',
        desc=
        'the regularisation parameter lambda that controls the strength of the constraint (default = 1.0).'
    )
    maximum_harmonic_order = traits.Int(
        argstr='-lmax %s',
        desc=
        'set the maximum harmonic order for the output series. By default, the program will use the highest possible lmax given the number of diffusion-weighted images.'
    )
    threshold_value = traits.Float(
        argstr='-threshold %s',
        desc=
        'the threshold below which the amplitude of the FOD is assumed to be zero, expressed as a fraction of the mean value of the initial FOD (default = 0.1)'
    )
    iterations = traits.Int(
        argstr='-niter %s',
        desc=
        'the maximum number of iterations to perform for each voxel (default = 50)'
    )
    debug = traits.Bool(argstr='-debug', desc='Display debugging messages.')
    directions_file = File(
        exists=True,
        argstr='-directions %s',
        position=-2,
        desc=
        'a text file containing the [ el az ] pairs for the directions: Specify the directions over which to apply the non-negativity constraint (by default, the built-in 300 direction set is used)'
    )

    normalise = traits.Bool(
        argstr='-normalise',
        position=3,
        desc="normalise the DW signal to the b=0 image")


class ConstrainedSphericalDeconvolutionOutputSpec(TraitedSpec):
    spherical_harmonics_image = File(
        exists=True, desc='Spherical harmonics image')


class ConstrainedSphericalDeconvolution(CommandLine):
    """
    Perform non-negativity constrained spherical deconvolution.

    Note that this program makes use of implied symmetries in the diffusion profile.
    First, the fact the signal attenuation profile is real implies that it has conjugate symmetry,
    i.e. Y(l,-m) = Y(l,m)* (where * denotes the complex conjugate). Second, the diffusion profile should be
    antipodally symmetric (i.e. S(x) = S(-x)), implying that all odd l components should be zero.
    Therefore, this program only computes the even elements. 	Note that the spherical harmonics equations used here
    differ slightly from those conventionally used, in that the (-1)^m factor has been omitted. This should be taken
    into account in all subsequent calculations. Each volume in the output image corresponds to a different spherical
    harmonic component, according to the following convention:

    * [0] Y(0,0)
    * [1] Im {Y(2,2)}
    * [2] Im {Y(2,1)}
    * [3] Y(2,0)
    * [4] Re {Y(2,1)}
    * [5] Re {Y(2,2)}
    * [6] Im {Y(4,4)}
    * [7] Im {Y(4,3)}

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> csdeconv = mrt.ConstrainedSphericalDeconvolution()
    >>> csdeconv.inputs.in_file = 'dwi.mif'
    >>> csdeconv.inputs.encoding_file = 'encoding.txt'
    >>> csdeconv.run()                                          # doctest: +SKIP
    """

    _cmd = 'csdeconv'
    input_spec = ConstrainedSphericalDeconvolutionInputSpec
    output_spec = ConstrainedSphericalDeconvolutionOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['spherical_harmonics_image'] = self.inputs.out_filename
        if not isdefined(outputs['spherical_harmonics_image']):
            outputs['spherical_harmonics_image'] = op.abspath(
                self._gen_outfilename())
        else:
            outputs['spherical_harmonics_image'] = op.abspath(
                outputs['spherical_harmonics_image'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_CSD.mif'


class EstimateResponseForSHInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-3,
        desc='Diffusion-weighted images')
    mask_image = File(
        exists=True,
        mandatory=True,
        argstr='%s',
        position=-2,
        desc=
        'only perform computation within the specified binary brain mask image'
    )
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output filename')
    encoding_file = File(
        exists=True,
        argstr='-grad %s',
        mandatory=True,
        position=1,
        desc=
        'Gradient encoding, supplied as a 4xN text file with each line is in the format [ X Y Z b ], where [ X Y Z ] describe the direction of the applied gradient, and b gives the b-value in units (1000 s/mm^2). See FSL2MRTrix'
    )
    maximum_harmonic_order = traits.Int(
        argstr='-lmax %s',
        desc=
        'set the maximum harmonic order for the output series. By default, the program will use the highest possible lmax given the number of diffusion-weighted images.'
    )
    normalise = traits.Bool(
        argstr='-normalise', desc='normalise the DW signal to the b=0 image')
    quiet = traits.Bool(
        argstr='-quiet',
        desc='Do not display information messages or progress status.')
    debug = traits.Bool(argstr='-debug', desc='Display debugging messages.')


class EstimateResponseForSHOutputSpec(TraitedSpec):
    response = File(exists=True, desc='Spherical harmonics image')


class EstimateResponseForSH(CommandLine):
    """
    Estimates the fibre response function for use in spherical deconvolution.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> estresp = mrt.EstimateResponseForSH()
    >>> estresp.inputs.in_file = 'dwi.mif'
    >>> estresp.inputs.mask_image = 'dwi_WMProb.mif'
    >>> estresp.inputs.encoding_file = 'encoding.txt'
    >>> estresp.run()                                   # doctest: +SKIP
    """
    _cmd = 'estimate_response'
    input_spec = EstimateResponseForSHInputSpec
    output_spec = EstimateResponseForSHOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['response'] = self.inputs.out_filename
        if not isdefined(outputs['response']):
            outputs['response'] = op.abspath(self._gen_outfilename())
        else:
            outputs['response'] = op.abspath(outputs['response'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_ER.txt'


def concat_files(bvec_file, bval_file, invert_x, invert_y, invert_z):
    bvecs = np.loadtxt(bvec_file)
    bvals = np.loadtxt(bval_file)
    if np.shape(bvecs)[0] > np.shape(bvecs)[1]:
        bvecs = np.transpose(bvecs)
    if invert_x:
        bvecs[0, :] = -bvecs[0, :]
        iflogger.info('Inverting b-vectors in the x direction')
    if invert_y:
        bvecs[1, :] = -bvecs[1, :]
        iflogger.info('Inverting b-vectors in the y direction')
    if invert_z:
        bvecs[2, :] = -bvecs[2, :]
        iflogger.info('Inverting b-vectors in the z direction')
    iflogger.info(np.shape(bvecs))
    iflogger.info(np.shape(bvals))
    encoding = np.transpose(np.vstack((bvecs, bvals)))
    _, bvec, _ = split_filename(bvec_file)
    _, bval, _ = split_filename(bval_file)
    out_encoding_file = bvec + '_' + bval + '.txt'
    np.savetxt(out_encoding_file, encoding)
    return out_encoding_file


class FSL2MRTrixInputSpec(TraitedSpec):
    bvec_file = File(
        exists=True, mandatory=True, desc='FSL b-vectors file (3xN text file)')
    bval_file = File(
        exists=True, mandatory=True, desc='FSL b-values file (1xN text file)')
    invert_x = traits.Bool(
        False, usedefault=True, desc='Inverts the b-vectors along the x-axis')
    invert_y = traits.Bool(
        False, usedefault=True, desc='Inverts the b-vectors along the y-axis')
    invert_z = traits.Bool(
        False, usedefault=True, desc='Inverts the b-vectors along the z-axis')
    out_encoding_file = File(genfile=True, desc='Output encoding filename')


class FSL2MRTrixOutputSpec(TraitedSpec):
    encoding_file = File(
        desc=
        'The gradient encoding, supplied as a 4xN text file with each line is in the format [ X Y Z b ], where [ X Y Z ] describe the direction of the applied gradient'
        'and b gives the b-value in units (1000 s/mm^2).')


class FSL2MRTrix(BaseInterface):
    """
    Converts separate b-values and b-vectors from text files (FSL style) into a
    4xN text file in which each line is in the format [ X Y Z b ], where [ X Y Z ]
    describe the direction of the applied gradient, and b gives the
    b-value in units (1000 s/mm^2).

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> fsl2mrtrix = mrt.FSL2MRTrix()
    >>> fsl2mrtrix.inputs.bvec_file = 'bvecs'
    >>> fsl2mrtrix.inputs.bval_file = 'bvals'
    >>> fsl2mrtrix.inputs.invert_y = True
    >>> fsl2mrtrix.run()                                # doctest: +SKIP
    """
    input_spec = FSL2MRTrixInputSpec
    output_spec = FSL2MRTrixOutputSpec

    def _run_interface(self, runtime):
        encoding = concat_files(self.inputs.bvec_file, self.inputs.bval_file,
                                self.inputs.invert_x, self.inputs.invert_y,
                                self.inputs.invert_z)
        return runtime

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['encoding_file'] = op.abspath(
            self._gen_filename('out_encoding_file'))
        return outputs

    def _gen_filename(self, name):
        if name == 'out_encoding_file':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, bvec, _ = split_filename(self.inputs.bvec_file)
        _, bval, _ = split_filename(self.inputs.bval_file)
        return bvec + '_' + bval + '.txt'


class GenerateDirectionsInputSpec(CommandLineInputSpec):
    num_dirs = traits.Int(
        mandatory=True,
        argstr='%s',
        position=-2,
        desc='the number of directions to generate.')

    power = traits.Float(
        argstr='-power %s',
        desc='specify exponent to use for repulsion power law.')
    niter = traits.Int(
        argstr='-niter %s',
        desc='specify the maximum number of iterations to perform.')
    display_info = traits.Bool(
        argstr='-info', desc='Display information messages.')
    quiet_display = traits.Bool(
        argstr='-quiet',
        desc='do not display information messages or progress status.')
    display_debug = traits.Bool(
        argstr='-debug', desc='Display debugging messages.')
    out_file = File(
        name_source=['num_dirs'],
        name_template='directions_%d.txt',
        argstr='%s',
        hash_files=False,
        position=-1,
        desc='the text file to write the directions to, as [ az el ] pairs.')


class GenerateDirectionsOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='directions file')


class GenerateDirections(CommandLine):
    """
    generate a set of directions evenly distributed over a hemisphere.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> gendir = mrt.GenerateDirections()
    >>> gendir.inputs.num_dirs = 300
    >>> gendir.run()                                          # doctest: +SKIP
    """

    _cmd = 'gendir'
    input_spec = GenerateDirectionsInputSpec
    output_spec = GenerateDirectionsOutputSpec


class FindShPeaksInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-3,
        desc='the input image of SH coefficients.')
    directions_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='the set of directions to use as seeds for the peak finding')
    peaks_image = File(
        exists=True,
        argstr='-peaks %s',
        desc=
        'the program will try to find the peaks that most closely match those in the image provided'
    )
    num_peaks = traits.Int(
        argstr='-num %s', desc='the number of peaks to extract (default is 3)')
    peak_directions = traits.List(
        traits.Float,
        argstr='-direction %s',
        sep=' ',
        minlen=2,
        maxlen=2,
        desc=
        'phi theta.  the direction of a peak to estimate. The algorithm will attempt to find the same number of peaks as have been specified using this option '
        ' phi: the azimuthal angle of the direction (in degrees). theta: the elevation angle of the direction (in degrees, from the vertical z-axis)'
    )
    peak_threshold = traits.Float(
        argstr='-threshold %s',
        desc=
        'only peak amplitudes greater than the threshold will be considered')
    display_info = traits.Bool(
        argstr='-info', desc='Display information messages.')
    quiet_display = traits.Bool(
        argstr='-quiet',
        desc='do not display information messages or progress status.')
    display_debug = traits.Bool(
        argstr='-debug', desc='Display debugging messages.')
    out_file = File(
        name_template="%s_peak_dirs.mif",
        keep_extension=False,
        argstr='%s',
        hash_files=False,
        position=-1,
        desc=
        'the output image. Each volume corresponds to the x, y & z component of each peak direction vector in turn',
        name_source=["in_file"])


class FindShPeaksOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='Peak directions image')


class FindShPeaks(CommandLine):
    """
    identify the orientations of the N largest peaks of a SH profile

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> shpeaks = mrt.FindShPeaks()
    >>> shpeaks.inputs.in_file = 'csd.mif'
    >>> shpeaks.inputs.directions_file = 'dirs.txt'
    >>> shpeaks.inputs.num_peaks = 2
    >>> shpeaks.run()                                          # doctest: +SKIP
    """

    _cmd = 'find_SH_peaks'
    input_spec = FindShPeaksInputSpec
    output_spec = FindShPeaksOutputSpec


class Directions2AmplitudeInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc=
        'the input directions image. Each volume corresponds to the x, y & z component of each direction vector in turn.'
    )
    peaks_image = File(
        exists=True,
        argstr='-peaks %s',
        desc=
        'the program will try to find the peaks that most closely match those in the image provided'
    )
    num_peaks = traits.Int(
        argstr='-num %s', desc='the number of peaks to extract (default is 3)')
    peak_directions = traits.List(
        traits.Float,
        argstr='-direction %s',
        sep=' ',
        minlen=2,
        maxlen=2,
        desc=
        'phi theta.  the direction of a peak to estimate. The algorithm will attempt to find the same number of peaks as have been specified using this option '
        ' phi: the azimuthal angle of the direction (in degrees). theta: the elevation angle of the direction (in degrees, from the vertical z-axis)'
    )
    display_info = traits.Bool(
        argstr='-info', desc='Display information messages.')
    quiet_display = traits.Bool(
        argstr='-quiet',
        desc='do not display information messages or progress status.')
    display_debug = traits.Bool(
        argstr='-debug', desc='Display debugging messages.')
    out_file = File(
        name_template="%s_amplitudes.mif",
        keep_extension=False,
        argstr='%s',
        hash_files=False,
        position=-1,
        desc='the output amplitudes image',
        name_source=["in_file"])


class Directions2AmplitudeOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='amplitudes image')


class Directions2Amplitude(CommandLine):
    """
    convert directions image to amplitudes

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> amplitudes = mrt.Directions2Amplitude()
    >>> amplitudes.inputs.in_file = 'peak_directions.mif'
    >>> amplitudes.run()                                          # doctest: +SKIP
    """

    _cmd = 'dir2amp'
    input_spec = Directions2AmplitudeInputSpec
    output_spec = Directions2AmplitudeOutputSpec
